﻿using System;
using System.IO;
using Android.Content;
using Java.IO;
using Xamarin.Forms;
using System.Threading.Tasks;
using Test.Droid;
using Android.Support.V4.Content;
using Android.Graphics;

[assembly: Xamarin.Forms.Dependency(typeof(SaveAndView_Android))]
namespace Test.Droid
{
    public class SaveAndView_Android : ISaveAndView
    {
        public async Task SaveAndView(string fileName, string contentType, MemoryStream stream)
        {
            try
            {
                string root = null;

                //Get the root path of android device.
                if (Android.OS.Environment.IsExternalStorageEmulated)
                {
                    root = Android.OS.Environment.ExternalStorageDirectory.ToString();
                }
                else
                    root = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);

                //Create directory and file.
                Java.IO.File myDir = new Java.IO.File(root + "/Test");
                myDir.Mkdir();

                Java.IO.File file = new Java.IO.File(myDir, fileName);

                //Remove the file if exists.
                if (file.Exists()) file.Delete();

                //Write the stream into file.
                FileOutputStream outs = new FileOutputStream(file);
                outs.Write(stream.ToArray());

                outs.Flush();
                outs.Close();

                //Launch the saved file for viewing in default viewer.
                if (file.Exists())
                {
                    //Android.Net.Uri path = Android.Net.Uri.FromFile(file);
                    string extension = Android.Webkit.MimeTypeMap.GetFileExtensionFromUrl(Android.Net.Uri.FromFile(file).ToString());
                    string mimeType = Android.Webkit.MimeTypeMap.Singleton.GetMimeTypeFromExtension(extension);
                    Intent intent = new Intent(Intent.ActionView);
                    intent.SetFlags(ActivityFlags.ClearTop | ActivityFlags.NewTask);
                    Android.Net.Uri path = FileProvider.GetUriForFile(Forms.Context, Android.App.Application.Context.PackageName + ".fileprovider", file);
                    intent.SetDataAndType(path, mimeType);
                    intent.AddFlags(ActivityFlags.GrantReadUriPermission);
                    Forms.Context.StartActivity(Intent.CreateChooser(intent, "Choose App"));
                }
            }
            catch (Exception ex)
            {
            }
        }

        public async Task<bool> SaveImage(string path, ImageSource imageData)
        {
            try
            {
                var imagem = await this.GetImage(imageData);
                using (System.IO.FileStream fs = new System.IO.FileStream(path, System.IO.FileMode.OpenOrCreate))
                {
                    imagem.Compress(Bitmap.CompressFormat.Jpeg, 100, fs);
                    return await Task.FromResult<bool>(true);
                }
            }
            catch (Exception ex)
            {
                return await Task.FromResult<bool>(false);
            }
        }

        private async Task<Bitmap> GetImage(ImageSource imageData)
        {
            Bitmap result = null;
            try
            {
                var renderer = imageData.GetHandler();
                result = await renderer.LoadImageAsync(imageData, Forms.Context);
                return result;
            }
            catch (Exception ex)
            {
                return result;
            }
        }
    }
}