﻿using Plugin.Media;
using Plugin.Media.Abstractions;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace Test
{
    public class MainPageViewModel
    {
        private Random random = new Random();

        public Command ReportCommand { get; set; }

        public Command AddDataCommand { get; set; }

        public ObservableCollection<SiteData> DataCollection { get; set; }

        public MainPageViewModel()
        {
            this.ReportCommand = new Command(this.GenerateReport, ()=>this.DataCollection.Count > 0);
            this.AddDataCommand = new Command(this.AddData);
            this.DataCollection = new ObservableCollection<SiteData>();
        }

        private void GenerateReport()
        {
            foreach (var item in this.DataCollection)
            {
                var fullPath = Path.Combine(App.ImagesPath, item.ImageFileName);
                var bytes = this.GetImageBytes(fullPath);
                item.ImageData = bytes;
            }

            ReportGenerator.GenerateReport(this.DataCollection);
        }

        private byte[] GetImageBytes(string imagePath)
        {
            using (var streamReader = new StreamReader(imagePath))
            {
                var bytes = default(byte[]);
                using (var memStream = new MemoryStream())
                {
                    streamReader.BaseStream.CopyTo(memStream);
                    bytes = memStream.ToArray();
                }

                return bytes;
            }
        }

        private async void AddData()
        {
            // Capture image
            var imageData = await this.GetPhoto();
            if (imageData == null)
            {
                return;
            }
            
            var randomNo = random.Next();
            var data = new SiteData(randomNo, $"Random: {randomNo}")
            {
                DateTime = DateTime.Now,
                ImageFileName = imageData
            };

            this.DataCollection.Add(data);

            ((Command)this.ReportCommand).ChangeCanExecute();
        }

        private async Task<string> GetPhoto()
        {
            await CrossMedia.Current.Initialize();

            if (!CrossMedia.Current.IsCameraAvailable || !CrossMedia.Current.IsTakePhotoSupported)
            {
                return null;
            }

            try
            {
                using (var file = await CrossMedia.Current.TakePhotoAsync(new Plugin.Media.Abstractions.StoreCameraMediaOptions
                {
                    PhotoSize = PhotoSize.Medium
                }))
                {
                    if (file == null)
                    {
                        return null;
                    }

                    var imageFileName = file.Path.Split('/').Last();
                    var imageSource = ImageSource.FromStream(() =>
                    {
                        var stream = file.GetStream();
                        return stream;
                    });

                    var imageFullPath = Path.Combine(App.ImagesPath, imageFileName);
                    await DependencyService.Get<ISaveAndView>().SaveImage(imageFullPath, imageSource);
                    return imageFileName;
                }
            }
            catch (Exception ex)
            {

                throw;
            }
        }
    }
}
